"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.S3CustomStorage = void 0;
const client_s3_1 = require("@aws-sdk/client-s3");
class S3WithPartsStorage {
    constructor({ region, secret, key }) {
        this.s3 = null;
        this.s3 = new client_s3_1.S3({
            region: process.env.AWS_ACCESS_REGION || region,
            credentials: {
                accessKeyId: process.env.AWS_ACCESS_KEY_ID || key,
                secretAccessKey: process.env.AWS_ACCESS_KEY_SECRET || secret,
            },
        });
    }
    async _handleFile(_req, file, callback) {
        if (typeof file.fieldname === "string") {
            try {
                file.fieldname = JSON.parse(file.fieldname);
            }
            catch (err) {
                file.fieldname = {};
            }
        }
        const uploadInfo = {
            id: null,
            filename: null,
            size: file.size || 0,
            chunks: 0,
            metadata: null,
            processing: false,
            data: file.fieldname,
            parts: [],
        };
        const promisePartsList = [];
        uploadInfo.filename = `${uploadInfo.data.clientHash ? `${uploadInfo.data.clientHash}/` : ''}${file.originalname}`;
        const cmuco = await this.createHeatmapToStorage(uploadInfo);
        uploadInfo.id = cmuco.UploadId;
        uploadInfo.metadata = cmuco.$metadata;
        console.info('Starting upload of %s', file.originalname);
        file.stream.on('readable', () => {
            if (uploadInfo.processing) {
                return;
            }
            return file.stream.read(9e6);
        });
        file.stream.on('data', (chunk) => {
            uploadInfo.processing = true;
            const actualChunk = uploadInfo.chunks + 1;
            uploadInfo.chunks = actualChunk;
            console.info('Uploading part %s of [ %s ]', actualChunk, file.originalname);
            uploadInfo.size = file.size || +uploadInfo.size + chunk.length;
            promisePartsList.push(new Promise(async (resolve) => {
                const uploadedResult = await this.uploadHeatmapToStorage(chunk, uploadInfo, actualChunk);
                console.info('Uploading completed of part %s of [ %s ]', actualChunk, file.originalname);
                return resolve({
                    PartNumber: actualChunk,
                    ETag: uploadedResult.ETag,
                });
            }));
            uploadInfo.processing = false;
            file.stream.emit('readable');
        });
        file.stream.on('end', async () => {
            uploadInfo.parts = await Promise.all(promisePartsList);
            await this.completeStorageUpload(uploadInfo);
            console.info('Upload of [ %s ] is completed!', file.originalname);
            delete uploadInfo.processing;
            return callback(null, uploadInfo);
        });
    }
    async createHeatmapToStorage(info) {
        return this.s3.createMultipartUpload({
            Bucket: process.env.AWS_BUCKET_NAME,
            Key: `${info.filename}`,
        });
    }
    async uploadHeatmapToStorage(file, info, partNumber) {
        return this.s3.uploadPart({
            Bucket: process.env.AWS_BUCKET_NAME,
            Key: `${info.filename}`,
            PartNumber: partNumber,
            UploadId: info.id,
            Body: file,
        });
    }
    async completeStorageUpload(info) {
        return this.s3.completeMultipartUpload({
            Bucket: process.env.AWS_BUCKET_NAME,
            Key: `${info.filename}`,
            UploadId: info.id,
            MultipartUpload: {
                Parts: info.parts,
            },
        });
    }
    async abortStorageUpload(info) {
        return this.s3.abortMultipartUpload({
            Bucket: process.env.AWS_BUCKET_NAME,
            Key: `${info.filename}`,
            UploadId: info.id,
        });
    }
}
function S3CustomStorage(opts) {
    return new S3WithPartsStorage(opts);
}
exports.S3CustomStorage = S3CustomStorage;
//# sourceMappingURL=s3.storage.js.map